Scriptname GEmotionStore extends Alias

int joy = 50
int sadness = 50
int trust = 50
int disgust = 50
int fear = 50
int anger = 50
int anticipation = 50
int surprise = 50

int[] maxValEmotion

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: SetEquilibriumEmotions
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function should be called when an NPC is initialized to set the equilibrium values for each emotion pair. If a set of equilibrium 
;; values does not total 100, the default values of 50 are used.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Function SetEquilibriumEmotions(int j, int sad, int t, int d, int f, int ang, int ant, int sur)
	if (j + sad == 100)
		joy = j
		sadness = sad
	endif
	
	if (t + d == 100)
		trust = t
		disgust = d
	endif
	
	if (f + ang == 100)
		fear = f
		anger = ang
	endif
	
	if (ant + sur == 100)
		anticipation = ant
		surprise = sur
	endif
	
	return
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: UpdateEmotions
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in delta emotion values and updates the NPC's stored emotion values. Delta values can be the result of appraisals or
;; decay functions. It called the CheckCap function to ensure that changes do not result in values < 0 or > 100.
;; A border case exists where the passed in deltas in a pair are equal and opposite which causes resulting changes to double. This is accounted 
;; for with a simple equality check.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Function UpdateEmotions(int j, int sad, int t, int d, int f, int ang, int ant, int sur)
	;; Joy and Sadness
	if ((-1 * j) == sad)
		joy = CheckCap(joy + j)
		sadness = CheckCap(sadness - j)
	else
		joy = CheckCap(joy + (j - sad))
		sadness = CheckCap(sadness + (sad - j))
	endif
	
	;; Trust and Disgust
	if ((-1 * t) == d)
		trust = CheckCap(trust + t)
		disgust = CheckCap(disgust - t)
	else
		trust = CheckCap(trust + (t - d))
		disgust = CheckCap(disgust + (d - t))
	endif
		
	;; Fear and Anger
	if ((-1 * f) == ang)
		fear = CheckCap(fear + f)
		anger = CheckCap(anger - f)
	else
		fear = CheckCap(fear + f - ang)
		anger = CheckCap(anger + ang - f)
	endif
	
	;; Anticipation and Surprise
	if ((-1 * ant) == sur)
		anticipation = CheckCap(anticipation + ant)
		surprise = CheckCap(surprise - ant)
	else
		anticipation = CheckCap(anticipation + (ant - sur))
		surprise = CheckCap(surprise + (sur - ant))
	endif
	
	return
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: CheckCap
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in a value and checks if it is in the range [0, 100]. If a value does not meet a condition, the function returns the value used to
;; check the change (0 or 100). Otherwise it returns the passed in value.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int Function CheckCap(int ch)
	if (ch > 100)
		ch = 100	
	elseif (ch < 0)
		ch = 0
	endif
	
	return ch		
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: UpdatemaxValEmotion
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function can be run to evaluate and store the largest emotion value.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Function UpdatemaxValEmotion()
	int[] maxEm = new int [2]
	int maxVal = -1

	if (maxVal < joy)
		maxEm[0] = 0
		maxEm[1] = joy
		maxVal = joy
	endif
	
	if (maxVal < sadness)
		maxEm[0] = 1
		maxEm[1] = sadness
		maxVal = sadness
	endif
	
	if (maxVal < trust)
		maxEm[0] = 2
		maxEm[1] = trust
		maxVal = trust
	endif
	
	if (maxVal < disgust)
		maxEm[0] = 3
		maxEm[1] = disgust
		maxVal = disgust
	endif
	
	if (maxVal < fear)
		maxEm[0] = 5
		maxEm[1] = fear
		maxVal = fear
	endif
	
	if (maxVal < anger)
		maxEm[0] = 4
		maxEm[1] = anger
		maxVal = anger
	endif
	
	if (maxVal < anticipation)
		maxEm[0] = 6
		maxEm[1] = anticipation
		maxVal = anticipation
	endif
	
	if (maxVal < surprise)
		maxEm[0] = 7
		maxEm[1] = surprise
		maxVal = surprise
	endif
	
	maxValEmotion = maxEm
	
	return
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Max Emotion Functions
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; These functions can be used to retrieve the emotion with the largest value.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int[] Function GetMaxEmotion()
	return maxValEmotion
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Get Emotion Functions
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; These functions return current values for individual emotions.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int Function GetJoy()
	return joy
endfunction

int Function GetSadness()
	return sadness
endfunction

int Function GetTrust()
	return trust
endfunction

int Function GetDisgust()
	return disgust
endfunction

int Function GetFear()
	return fear
endfunction

int Function GetAnger()
	return anger
endfunction

int Function GetAnticipation()
	return anticipation
endfunction

int Function GetSurprise()
	return surprise
endfunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetValues
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function, created for debugging purposes, returns a string representing the current values saved in the emotion store. 
;; ---------------------------------------------------------------------------------------------------------------------------------------------
String Function GetValues()
	return ("Joy = " + GetJoy() + ", Sadness = " + GetSadness() + "\nTrust = " + GetTrust() + ", Disgust = " + GetDisgust() + "\nFear = " + GetFear() + ", Anger = " + GetAnger() + "\nAnticipation = " + GetAnticipation() + ", Surprise = " + GetSurprise())
EndFunction