Scriptname GGoals extends Alias

string[] Property goals Auto
float[] Property goalUtilities Auto ;; [0, 1]
float[] Property goalResponsibility Auto ;; [0, 1]

string[] Property stimuli Auto ;; Must match GLaDOS keywords
string[] Property affectsGoal Auto ;; must be an item in the goals array
float[] Property eventCongruence Auto ;; [-1, 1]
float[] Property stimuliFamiliarity Auto ;; [-1, 1]

string[] Property item Auto ;; Stress related items
float[] Property stressModifier Auto ;; [-1, 1]

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetGoalInformation
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function finds the goal associated with a stimuli and returns all relevant information pertaining to that goal by calling 
;; CompileGoalInformation.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
float[] Function GetGoalInformation(string stimuli)
	return CompileGoalInformation(FindStimuliIndex(stimuli))
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetGoalInformationByStimuliIndex
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function is the same as GetGoalInformation, but it takes in a specific stimuli index instead of searching for it.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
float[] Function GetGoalInformationByStimuliIndex(int stimIndex)
	return CompileGoalInformation(stimIndex)
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: CompileGoalInformation
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function finds gathers all values associated with a goal and stimuli pair:
;; 0 = Goal utility (importance to NPC) [0, 1]
;; 1 = How much the NPC feels responsible for the goal's success [-1, 1]
;; 2 = Stimuli congruence with goal [-1, 1]
;; 3 = The NPC's familiarity (positive or negative) with the stimuli [-1, 1]
;; It is used by GetGoalInformation and GetGoalInformationByStimuliIndex. 
;; ---------------------------------------------------------------------------------------------------------------------------------------------
float[] Function CompileGoalInformation(int stimIndex)
	float[] goalInfo = new float [4]
	int goalIndex = FindGoalIndex(affectsGoal[stimIndex])
	
	if (stimIndex > -1)
		goalInfo[0] = goalUtilities[goalIndex]
		goalInfo[1] = goalResponsibility[goalIndex]
		
		goalInfo[2] = eventCongruence[stimIndex]
		goalInfo[3] = stimuliFamiliarity[stimIndex]		
	else
		goalInfo[0] = 0
		goalInfo[1] = 0
		goalInfo[2] = 0
		goalInfo[3] = 0
	endif
	
	return goalInfo	
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: FindGoalIndex
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function finds the index number of the passed goal name. This index is used to access the associated information in the goalUtilities 
;; and goalResolves arrays.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int Function FindGoalIndex(string goalName)
	int i = 0
	int index = -1
	while (i < goals.Length)
		if (goals[i] == goalName)
			index = i
			i = goals.Length
		endif
		i = i + 1
	EndWhile
	
	return index
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: FindStimuliIndex
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function finds the string in the stimuli array and returns the index. The index is used to access stimuli information in the affectsGoal
;; and goalAlignment arrays. If the stimuli cannot be found, an index of -1 is returned.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int Function FindStimuliIndex(string tag)
	int index = -1
	
	int i = 0
	while (i < stimuli.Length)
		if (stimuli[i] == tag)
			index = i
			i = stimuli.Length
		endif
		i = i + 1
	EndWhile
	
	return index
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetItemInformation
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function finds the stress modifying value associated with an NPC's items, if it exists.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
float[] Function GetItemInformation (string[] itemNames)
	float[] temp = new float[15] ;; Purposefully made too large
	int count = 0
	int i = 0
	
	while ( i < itemNames.Length )
		int j = 0
		while ( j < item.Length )
			if ( itemNames[i] == item [j])
				temp[count] = stressModifier[j]
				count = count + 1
				j = item.Length  ;; break
			endif
			j = j + 1
		endwhile
		i = i + 1
	endwhile
	
	;; Create an array just big enough to hold all values
	float[] modifiers = Utility.CreateFloatArray(count)
	i = 0
	while ( i < count )
		modifiers[i] = temp[i]
		i = i + 1
	endwhile
	
	return modifiers
EndFunction