Scriptname GPlayer extends ReferenceAlias

GPlayerProfile selfProfile

bool isProcessing = false
bool hasStarted = false

bool unequipped = false
bool switching = false

int equipCounter = 0

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: OnInit
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function is called when this object is created (part of the Creation Kit library). It is used to set the player's profile object and 
;; perform the initial analysis.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Event OnInit()
	selfProfile = (self as Alias) as GPlayerProfile
	SelfAnalyse(GetReference() as Actor)
	hasStarted = true
		
	return
EndEvent

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Events: OnObjectEquipped and OnObjectUnequipped
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; These events, native to the Creation Kit, are triggered when the player changes their equipment. They ensure that the information in the 
;; player's profile are maintained.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Event OnObjectUnequipped(Form akBaseObject, ObjectReference akReference)
	;; In the case of switching equipment, this event is fired first. To 
	;; prevent the player's equipment from being analysed twice, tell this function
	;; to wait and see if the equipped event is also fired as signalled by the
	;; switching flag
	
	;;Debug.Notification("Triggered Unequip event")
	
	unequipped = true
	Utility.WaitMenuMode(1.0) ;; One second
	if (!switching)
		;;Debug.Notification("Analysing in Unequip event")
		SelfAnalyse(GetReference() as Actor)
	endif
	
	return
EndEvent

Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
	if (unequipped)
		switching = true
		;;Debug.Notification("Set switching flag (Equip event)")
	endif
	
	equipCounter = equipCounter + 1 ;; Method for preventing SelfAnalyse from running during a bulk equip
	int equipNum = equipCounter
	Utility.WaitMenuMode(0.5)
	if (equipCounter == equipNum)
		SelfAnalyse(GetReference() as Actor)
		;;Debug.Notification("Analysing in Equip event")
		equipCounter = 0 ;; Reset
	endif
	switching = false
	
	return
EndEvent

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: AssignProfileObject
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function should be called on NPC creation. It accepts a GProfile object and assigns it to GSensoryInput. 
;; This function is required because the GProfile object is attached to the NPC so that it can be referenced.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Function AssignProfileObject (GPlayerProfile pro)
	selfProfile = pro		
	return
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetSelfProfile
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function is used by other in-game objects to retrieve this NPC`s GLaDOS information.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
GPlayerProfile Function GetSelfProfile()
	return selfProfile
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: isProcessingProfile
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function is used to check if this NPC is currently running the SelfAnalyse function, which updates the NPC`s GProfile object.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
bool Function isProcessingProfile()
	return isProcessing
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: SelfAnalyse
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function is used to populate the GProfile object with this NPC`s information. This is done to help reduce the amount of processing
;; required when another NPC sees this NPC. The NPC's Actor object must be explicitly passed in to be referenceable.
;; Calls the GetArmorTypes, GetArmorEnchantments, and GetWeaponTypes functions.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Function SelfAnalyse(Actor target)
	;; -------------------------------------
	;; We are starting work on selfProfile
	;; -------------------------------------
	isProcessing = true
	
	;; -------------------------------------
	;; Looping variables
	;; -------------------------------------
	int i
	int j

	;; -------------------------------------
	;; Arrays to hold tag names and total occurences; Index variable tracks where in the arrays
	;; we are to avoid overwriting information
	;; -------------------------------------
	string[] tags = new string[31]
	int[] counts = new int [31]
	int tagArrayIndex = 0
	
	;; -------------------------------------
	;; Temporary variables for processing tags
	;; -------------------------------------	
	string[] tempTags
	int foundTagIndex
	
	;; -------------------------------------
	;; Variables for holding item names, attack rating and defence rating
	;; -------------------------------------	
	string[] itemNames = new string[31]
	int itemIndex = 0
	int weaponRating = 0
	int armorRating = 0	
	
	;; -------------------------------------
	;; Process Armor and clothing -- this is completed one piece at a time so that three types
	;; of information can be collected (tags, armor rating, and item names).
	;; Shields are included at this stage.
	;; -------------------------------------
	Armor[] a = GetArmorPieces(target)
	int lim = a.Length
	i = 0
	while (i < lim)
		if(a[i]) ;; Ensure that the value is not None
			;; Find all of the GLaDOS tags associated with this item
			tempTags = GetArmorTypesSingle(a[i])
			j = 0
			while (j < tempTags.Length) ;; Should be 2				
				if (tempTags[j] != "") ;; Make sure the array value is not empty
					foundTagIndex = FindTagInList(tags, tempTags[j])
					
					if (foundTagIndex > -1)
						counts[foundTagIndex] = counts[foundTagIndex] + 1
						
						;; Double the tag if it covers the body
						if (a[i].isClothingBody() || a[i].isCuirass())
							counts[foundTagIndex] = counts[foundTagIndex] + 1
						endif
					else
						tags[tagArrayIndex] = tempTags[j]
						
						;; Double the tag if it covers the body
						if (a[i].isClothingBody() || a[i].isCuirass())
							counts[tagArrayIndex] = 2
						else
							counts[tagArrayIndex] = 1
						endif
						
						tagArrayIndex = tagArrayIndex + 1
					endif
				endif
				
				;; Goto next tag to examine
				j = j + 1
			endwhile
			
			;; Record armor rating and name
			armorRating = armorRating + a[i].GetArmorRating()
			
			itemNames[itemIndex] = a[i].GetName()
			itemIndex = itemIndex + 1
			
			i = i + 1
		else
			i = lim
		endif
	endwhile	
		
	;; -------------------------------------
	;; Process weapons and spells -- this is completed one piece at a time so that three types
	;; of information can be collected (tags, attack rating, and item names).
	;; -------------------------------------
	int attackRating = 0
	bool leftHandEmpty = false
	Weapon w
	Spell s
	string enchantmentName
	string weaponName
	
	;; -------------------------------------
	;; Analyse left hand first
	;; Items that require two hands to wield are skipped because their information can only
	;; be accessed by analysing the right hand (ID = 5, 6, 7, 12).
	;; -------------------------------------
	int itemType = target.GetEquippedItemType(0)
	;;Debug.Notification("Sensory Input (Left hand weapon value): " + itemType)

	;; Nothing equipped
	if (itemType == 0)
		leftHandEmpty = true
	
	;; Spell
	elseif (itemType == 9)
		s = target.GetEquippedSpell(0)
		
		tempTags[0] = "GLaDOS_Mage"
		if (isSpellConjurationSchool(s))
			tempTags[1] = "GLaDOS_Necromancy"
		else
			tempTags[1] = ""
		endif
		
		if (s.isHostile())
			attackRating = attackRating + Math.Ceiling(s.GetNthEffectMagnitude(0))
		endif
		
	;; Torch (can deal 3 fire damage for 1 second)
	elseif (itemType == 11)
		attackRating = attackRating + 3		
		
	else
		
		;; Single handed weapons
		if (itemType > 0 && itemType < 5)
			w = target.GetEquippedWeapon(true)
		
			tempTags = GetWeaponTags(w)
		
			attackRating = attackRating + w.GetBaseDamage()
		
			weaponName = w.GetName()
		
		;; Staves (have spells associated with them)
		elseif (itemType == 8)
			w = target.GetEquippedWeapon(true)
		
			tempTags[0] = "GLaDOS_Mage"
			enchantmentName = w.GetName()
			if ( StringUtil.Find(enchantmentName, "Zombie") > -1 || StringUtil.Find(enchantmentName, "Revenant") > -1 || StringUtil.Find(enchantmentName, "Reanimation") > -1 )
				tempTags[1] = "GLaDOS_Necromancy"
			elseif ( StringUtil.Find(enchantmentName, "Daedric") > -1 )
				tempTags[1] = "GLaDOS_Daedric"
			else
				tempTags[1] = ""
			endif
		
			;; Get the base damage of the staff and the enchantment on it (if the enchantment is considered hostile)
			attackRating = attackRating + w.GetBaseDamage()
			if (w.GetEnchantment().IsHostile())
				attackRating = attackRating + Math.Ceiling(w.GetEnchantment().GetNthEffectMagnitude(0))
			endif
		
			weaponName = w.GetName()
		endif
		
		if (weaponName != "")
			;; Record the name of the item
			itemNames[itemIndex] = weaponName
			itemIndex = itemIndex + 1
		endif
	endif
	
	;; Record tags for left hand item
	i = 0
	while (i < tempTags.Length)
		if (tempTags[i] != "")
			foundTagIndex = FindTagInList(tags, tempTags[i])
			if (foundTagIndex > -1)
				counts[foundTagIndex] = counts[foundTagIndex] + 1
			else
				tags[tagArrayIndex] = tempTags[i]
				counts[tagArrayIndex] = 1
				tagArrayIndex = tagArrayIndex + 1
			endif
		endif
		
		i = i + 1
	endWhile
		
	;;Debug.Notification("Name: " + itemNames[itemIndex - 1] + ", Tags: " + tempTags + ", Attack Rating: " + attackRating + ", Defence Rating: " + armorRating)
		
	;; -------------------------------------
	;; Analyse right hand next
	;; -------------------------------------
	;;Debug.Notification("Left hand item type = " + itemType)
	itemType = target.GetEquippedItemType(1)
	;;Debug.Notification("Right hand item type = " + itemType)
	
	tempTags[0] = ""
	tempTags[1] = ""
	weaponName = ""
	
	;; Both hands are empty, so we are unarmed
	;; Unarmed damage changes between races due to unique characteristics
	if (itemType == 0)
		if (leftHandEmpty)
			if (target.GetRace().GetName() == "Khajiit")
				attackRating = attackRating + 22
			elseif (target.GetRace().GetName() == "Argonian")
				attackRating = attackRating + 10
			else
				attackRating = attackRating + 4
			endif
		endif
		
	;; Spell
	elseif (itemType == 9)
		s = target.GetEquippedSpell(1)
		
		tempTags[0] = "GLaDOS_Mage"
		if (isSpellConjurationSchool(s))
			tempTags[1] = "GLaDOS_Necromancy"
		else
			tempTags[1] = ""
		endif
		
		if (s.isHostile())
			attackRating = attackRating + Math.Ceiling(s.GetNthEffectMagnitude(0))
		endif
		
	;; Torch (can deal 3 fire damage for 1 second)
	elseif (itemType == 11)
		attackRating = attackRating + 3
		
	else
	;; Single and two handed weapons
		if (itemType > 0 && itemType < 7)
			w = target.GetEquippedWeapon(false)
			
			tempTags = GetWeaponTags(w)
			
			attackRating = attackRating + w.GetBaseDamage()
			
			weaponName = w.GetName()
		
		;; Bow and crossbow
		elseif (itemType == 7 || itemType == 12)
			w = target.GetEquippedWeapon(false)
			
			tempTags = GetWeaponTags(w)
			
			attackRating = attackRating + w.GetBaseDamage()
			
			weaponName = w.GetName()
		
		;; Staves (have spells associated with them)
		elseif (itemType == 8)
			w = target.GetEquippedWeapon(false)
			
			tempTags[0] = "GLaDOS_Mage"
			enchantmentName = w.GetName()
			if ( StringUtil.Find(enchantmentName, "Zombie") > -1 || StringUtil.Find(enchantmentName, "Revenant") > -1 || StringUtil.Find(enchantmentName, "Reanimation") > -1 )
				tempTags[1] = "GLaDOS_Necromancy"
			elseif ( StringUtil.Find(enchantmentName, "Daedric") > -1 )
				tempTags[1] = "GLaDOS_Daedric"
			else
				tempTags[1] = ""
			endif
			
			;; Get the base damage of the staff and the enchantment on it (if the enchantment is considered hostile)
			attackRating = attackRating + w.GetBaseDamage()
			if (w.GetEnchantment().IsHostile())
				attackRating = attackRating + Math.Ceiling(w.GetEnchantment().GetNthEffectMagnitude(0))
			endif
			
			weaponName = w.GetName()
		endif		
		
		if (weaponName != "")
			;; Record the name of the item
			itemNames[itemIndex] = weaponName
			itemIndex = itemIndex + 1	
		endif
	endif
		
	;; -------------------------------------
	;; Record tags for right hand item (two-handed item tags are doubled)
	;; -------------------------------------
	i = 0
	while (i < tempTags.Length)
		if (tempTags[i] != "")
			foundTagIndex = FindTagInList(tags, tempTags[i])
			if (foundTagIndex > -1)
				counts[foundTagIndex] = counts[foundTagIndex] + 1			
				if ((itemType > 4 && itemType < 8) || itemType == 12)
					counts[foundTagIndex] = counts[foundTagIndex] + 1
				endif
			else
				tags[tagArrayIndex] = tempTags[i]
				
				counts[tagArrayIndex] = 1
				if ((itemType > 4 && itemType < 8) || itemType == 12)
					counts[tagArrayIndex] = counts[tagArrayIndex] + 1
				endif
				
				tagArrayIndex = tagArrayIndex + 1
			endif
		endif
		
		i = i + 1
	endWhile

	;; -------------------------------------
	;; Debugging
	;; -------------------------------------
	;;i = 0
	;;string dbg = ""
	;;while (i < tagArrayIndex)
	;;	dbg = dbg + tags[i] + ": " + counts[i] + ", "
	;;	i = i + 1
	;;endwhile
	;;Debug.MessageBox("Sensory Input tags: \n" + dbg + "\nTotal attack rating: " + attackRating + ", Total armor rating = " + armorRating)
	;;Debug.MessageBox("Item names: " + itemNames)
	
	;; -------------------------------------
	;; Fill out the player's profile object
	;; -------------------------------------		
	UpdateProfile(tags, tagArrayIndex, counts, attackRating, armorRating, itemNames, itemIndex)
	
	;;dbg = "Profile Owner: " + selfProfile.GetProfileOwnerName() + "\nRace: " + selfProfile.GetProfileOwnerRace() + "\nStimuli: " + selfProfile.GetStimuli() + "\nDedication: " + selfProfile.GetStimuliDedication() + "\nAttack Rating: " + selfProfile.GetAttackRating() + ", Defence Rating: " + selfProfile.GetDefenceRating() + "\nItem list: " + selfProfile.GetItemNames() + "\nTime Updated: "  + selfProfile.GetProfileCreationTime()
	;;Debug.MessageBox("GPlayerProfile Object Data:\n\n" + dbg)

	;; -------------------------------------
	;; We are finished working on selfProfile
	;; -------------------------------------
	isProcessing = false
	
	return
EndFunction 

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetArmorTypesSingle
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function is used by SelfAnalyse to extract GLaDOS tags from one piece of the NPC`s worn armor. It allows for a piece or armor to have at
;; most two GLaDOS tags.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
string[] Function GetArmorTypesSingle(Armor a)
	string[] types = new string[2]
	int counter = 0
	
	int keys = a.GetNumKeywords()
	int i = 0
	string kee
	
	while (i < keys && counter < 2)
		kee = a.GetNthKeyword(i).GetString()
		if (StringUtil.Find(kee, "GLaDOS") > -1)
			types[counter] = kee
			counter = counter + 1
		endif
		
		i = i + 1
	endwhile
	
	return types
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetArmorPieces
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in an actor to be searched and returns a list of all worned armor pieces.
;; This function is a modification of a Slot Masks example on the Creation Kit website
;; (http://www.creationkit.com/index.php?title=Slot_Masks_-_Armor)
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Armor[] Function GetArmorPieces (Actor target)
	Armor[] wornEnchantedForms = new Armor[10]
    int index
	int slotsChecked
    slotsChecked += 0x00100000
    slotsChecked += 0x00200000 ;ignore reserved slots
    slotsChecked += 0x80000000

	string armorPath
	int iFac
    int thisSlot = 0x01
    while (thisSlot < 0x80000000 && index < 10)
        if (Math.LogicalAnd(slotsChecked, thisSlot) != thisSlot) ;only check slots we haven't found anything equipped on already
            Armor thisArmor = target.GetWornForm(thisSlot) as Armor
            if (thisArmor)
				wornEnchantedForms[index] = thisArmor;;.getName()
				index += 1
                
                slotsChecked += thisArmor.GetSlotMask() ;add all slots this item covers to our slotsChecked variable
            else ;no armor was found on this slot
                slotsChecked += thisSlot
            endif
        endif
        thisSlot *= 2 ;double the number to move on to the next slot
    endWhile
	
    return wornEnchantedForms
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetWeaponTags
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in a weapon and extracts up to two GLaDOS tags.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
string[] Function GetWeaponTags(Weapon weap)
	string[] w = new string [2]
	int index = 0
	int keys = weap.GetNumKeywords()
	string current
	
	int i = 0
	while (i < keys && index < 2)
		current = weap.GetNthKeyword(i).GetString()
		if (StringUtil.Find(current, "GLaDOS") > -1)
			w[index] = current
			index = index + 1
		endif
			
		i = i + 1
	endwhile

	return w
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: isSpellConjurationSchool
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in a spell and determines if it is part of the School of Conjuration. This is used to distinguish between a Mage and a 
;; potential Necromancer (distinct GLaDOS tags).
;; ---------------------------------------------------------------------------------------------------------------------------------------------
bool Function isSpellConjurationSchool(Spell s)
	bool conjuration = false
	string perkName = s.GetPerk().GetName()
	
	if (StringUtil.Find(perkName, "Conjuration") > -1)
		conjuration = true
	endif
	
	return conjuration
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: FindTagInList
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; A helper function for identifying if the extracted tag type has already been found.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int Function FindTagInList (string[] list, string toFind)
	int found = -1
	int lim = list.Length
	
	int i = 0
	while (i < lim)
		if (list[i] == toFind)
			found = i
			i = lim
		endif
		
		i = i + 1
	endwhile
	
	return found
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: UpdateProfile
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function is used to update the player's profile object with the data extracted in SelfAnalyse.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Function UpdateProfile(string[] t, int tCount, int[] c, int attackRating, int armorRating, string[] it, int iCount)
	bool success
	
	;; First, resize the arrays so that they no longer contain null values
	string[] tags = Utility.CreateStringArray(tCount)
	int[] counts = Utility.CreateIntArray(tCount)
	int i = 0
	while (i < tCount)
		tags[i] = t[i]
		counts[i] = c[i]
		
		i = i + 1
	endwhile
	
	string[] items = Utility.CreateStringArray(iCount)
	i = 0
	while (i < iCount)
		items[i] = it[i]
	
		i = i + 1
	endwhile
	
	;; Fill in the profile object
	if (hasStarted)
		success = selfProfile.UpdateProfile(tags, counts, attackRating, armorRating, items)
	else
		success = selfProfile.CreateProfile((GetReference() as Actor), tags, counts, attackRating, armorRating, items)
	endif
	
	;;Debug.Notification("Successfully created profile? " + success)
	
	return
EndFunction