Scriptname GPlayerProfile extends Alias

Actor owner
string ownerName
string ownerRace

string[] stimuli
float[] stimuliDedication

int selfWeaponRating
int selfDefenceRating

string[] itemNames

float timestamp

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: CreateProfile
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function initializes all script values with the passed in values. Since the Actor associated with this script should never change, this
;; function should only be called once.
;; If the tags and tagCounts arrays are not the same length, this function returns "false". The only difference between initializing and updating
;; a profile object is the setting of the owner and ownerName values. Therefore, this function calls the UpdateProfile function to set the 
;; remaining  values.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
bool Function CreateProfile(Actor a, string[] tags, int[] tagCounts, int weaponRating, int defenceRating, string[] items)
	bool success = UpdateProfile(tags, tagCounts, weaponRating, defenceRating, items)

	if (success)
		owner = a
		ownerName = a.GetBaseObject().GetName()
		ownerRace = a.GetRace().GetName()
	endif	
	
	return success
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: UpdateProfile
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function should be used to update profile values when an Actor changes equipment. If the tags and tagCounts arrays are not the same 
;; length, this function returns "false".
;; ---------------------------------------------------------------------------------------------------------------------------------------------
bool Function UpdateProfile(string[] tags, int[] tagCounts, int weaponRating, int defenceRating, string[] items)
	bool success = false
	
	if (tags.Length == tagCounts.Length)
		stimuli = tags
		
		int i = 0
		int totalTags = 0
		while (i < tags.Length)
			totalTags = totalTags + tagCounts[i]
			i = i + 1
		endwhile
		
		;;Debug.Notification("Total tag count (GPlayerProfile object) = " + totalTags)
		
		i = 0
		stimuliDedication = Utility.CreateFloatArray(tags.Length)
		while (i < tags.Length)
			stimuliDedication[i] = (tagCounts[i] as float) / totalTags
			i = i + 1
		endwhile
		
		selfWeaponRating = weaponRating
		selfDefenceRating = defenceRating
				
		itemNames = items
		
		success = true
	endif
	
	timestamp = Utility.GetCurrentRealTime()
	
	return success
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetProfileCreationTime
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Returns the time (in seconds since the game launched) that this profile was updated.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
float Function GetProfileCreationTime()
	return timestamp
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetProfileOwner
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Returns the Actor that this profile describes.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Actor Function GetProfileOwner()
	return owner
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetProfileOwnerName
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Returns the Actor's name that this profile describes.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
string Function GetProfileOwnerName()
	return ownerName
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetProfileOwnerRace
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Returns the Actor's race.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
string Function GetProfileOwnerRace()
	return ownerRace
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetStimuli
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Returns the list of GLaDOS stimuli tags that this Actor has.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
string[] Function GetStimuli()
	return stimuli
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetStimuliDedication
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Returns a list of GLaDOS stimuli tags as a percentage representing how much of the Actor's total belongs are tagged with the same stimuli.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
float[] Function GetStimuliDedication()
	return stimuliDedication
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetAttackRating
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Returns the Actor's total weapon rating (max 56, 28 per
;; hand).
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int Function GetAttackRating()
	return selfWeaponRating
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetDefenceRating
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Returns the Actor's total defence rating (max 567).
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int Function GetDefenceRating()
	return selfDefenceRating
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetItemNames
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Returns the Actor's items as a list of names.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
string[] Function GetItemNames()
	return itemNames
EndFunction