Scriptname GPrimaryAppraisal extends Alias

bool Property pessimist = false Auto
bool Property lazy = false Auto
float Property emotionControl = 1.0 Auto
int Property energyProfile = 2 Auto ;; [0, 4]

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: MultiAppraise
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in the values for multiple stimuli, the goal that it affects, and some NPC personality traits and returns a set of floats
;; representing a coded basic emotion and its intensity.
;; The value for threatControl MUST BE NORMALIZED when passed to this function.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
float[] Function MultiAppraise(float[] goalUtility, float[] goalCongruence, float[] goalResponsibility, float[] stimuliFamiliarity, float threatControl, float[] stressMods)
	float[] ems = new float [2]
	int len = goalUtility.Length
	;;Debug.Notification("[GPrimaryAppraisal] Length of the utilties array = " + len)
	float[] dimTemp
	float[] dims = Utility.CreateFloatArray( len )

	;; Calculate the dimensions for each set of goal values and add it to a running total
	int i = 0
	while ( i < len )
		;;Debug.Notification("[GPrimaryAppraisal] While loop round " + i)

		;; Not all the values are 0, therefore we process it
		if ( goalUtility[i] != 0 && goalCongruence[i] != 0 && goalResponsibility[i] != 0 && stimuliFamiliarity[i] != 0 )
			;;Debug.Notification("[GPrimaryAppraisal] Calculating dimensions round " + i)
			dimTemp = CalculateDimensions( goalUtility[i], goalCongruence[i], goalResponsibility[i], stimuliFamiliarity[i], threatControl, stressMods )
			dims[0] = dims[0] + dimTemp[0]
			dims[1] = dims[1] + dimTemp[1]
			dims[2] = dims[2] + dimTemp[2]
			
			;;Debug.MessageBox("Dimensions this round (i = " + i + "): " + dims)

		;; There is no more useful information
		else
			i = len
		endif

		i = i + 1
	endwhile

	;; Average (normalize)
	dims[0] = dims[0] / len
	dims[1] = dims[1] / len
	dims[2] = dims[2] / len

	;; Find the emotion code and intensity
	ems[0] = ( MatchEmotion ( dims ) as float )
	ems[1] = CalculateIntensity ( dims )

	return ems
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: Appraise
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in the values about a stimuli, the goal that it affects, and some NPC personality traits and returns a set of floats
;; representing a coded basic emotion and its intensity.
;; The value for threatControl MUST BE NORMALIZED when passed to this function.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
float[] Function Appraise(float goalUtility, float goalCongruence, float goalResponsibility, float stimuliFamiliarity, float threatControl, float[] stressMods)
	float[] ems = new float [2]
	
	float[] dims = CalculateDimensions( goalUtility, goalCongruence, goalResponsibility, stimuliFamiliarity, threatControl, stressMods )
	
	ems[0] = ( MatchEmotion ( dims ) as float )
	ems[1] = CalculateIntensity ( dims )
	
	return ems
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: CalculateDimensions
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes all the gathered information about the NPC's perception and converts them into three appraisal dimensions: Desireability, 
;; Controllability, and Energy. These dimensions represent a vector space that is used to label the emotions and calculate their intensity. The 
;; dimension values are normalized.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
float[] Function CalculateDimensions(float goalUtility, float goalCongruence, float goalResponsibility, float stimuliFamiliarity, float threatControl, float[] stressMods)
	float[] dimensions = new float [3] ;; Desireability, Controllability, Energy
	
	;; Desireability (already normalized due to the range of goalUtility and goalCongruence)
	dimensions[0] = goalUtility * goalCongruence
	
	;; Controllability
	int numItems = stressMods.Length
	float predictionBias = goalResponsibility * stimuliFamiliarity
	int i = 0
	while (i < numItems)
		predictionBias = predictionBias + stressMods[i]
		i = i + 1
	endwhile
	predictionBias = predictionBias / (1.0 + numItems) ;; Normalizing value
	
	dimensions[1] = (2.0 / 3.14159265359) * Math.DegreesToRadians( Math.atan( emotionControl * ( ( threatControl + predictionBias ) / 2.0 ) ) )
	
	;; Effort
	dimensions[2] = (2.0 / 5.0) * goalUtility * ( ( (3.0 / 4.0) * (energyProfile - 2) ) + predictionBias )
	
	;;Debug.Notification("[GPrimaryAppraisal] Desireability = " + dimensions[0] + ", Controllability = " + dimensions[1] + ", Effort = " + dimensions[2])
	
	return dimensions
EndFunction


;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: MatchEmotion
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in an array of values and matches them to the dimension patterns of the basic emotions. It returns an integer coded 
;; emotion: 
;; 0 = Joy
;; 1 = Sadness
;; 2 = Trust
;; 3 = Disgust
;; 4 = Anger
;; 5 = Fear
;; 6 = Acceptance
;; 7 = Surprise
;; Dimensions must be passed in this order for correct pattern matching: Desireability, Controllability, Energy
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int Function MatchEmotion(float[] dimensions)
	int emotion
	
	if(dimensions[0] > 0 || (dimensions[0] == 0 && !pessimist))
		if(dimensions[1] > 0 || (dimensions[1] == 0 && !pessimist))
			if(dimensions[2] > 0 || (dimensions[2] == 0 && !lazy))
				emotion = 0 ;; Joy (+++)
			else
				emotion = 2 ;; Trust (++-)
			endif
		else
			if(dimensions[2] > 0 || (dimensions[2] == 0 && !lazy))
				emotion = 7 ;; Surprise (+-+)
			else
				emotion = 6 ;; Acceptance (+--)
			endif
		endif
	else
		if(dimensions[1] > 0 || (dimensions[1] == 0 && !pessimist))
			if(dimensions[2] > 0 || (dimensions[2] == 0 && !lazy))
				emotion = 4 ;; Anger (-++)
			else
				emotion = 3 ;; Disgust (-+-)
			endif
		else
			if(dimensions[2] > 0 || (dimensions[2] == 0 && !lazy))
				emotion = 5 ;; Fear (--+)
			else
				emotion = 1 ;; Sadness (---)
			endif
		endif
	endif
	
	return emotion
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: CalculateIntensity
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in an array of values and calculates the intensity of the vector that they represent as the vector's magnitude.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
float Function CalculateIntensity (float[] dimensions)
	return ( Math.sqrt( ( Math.pow(dimensions[0], 2) ) + ( Math.pow(dimensions[1], 2) ) + ( Math.pow(dimensions[2], 2) ) ) )
EndFunction